#!/usr/bin/env python
import setuptools  # noqa: F401
from pkg_resources import parse_version
from pathlib import Path
# from numpy.distutils.core import Extension, setup
import os
import subprocess

"""
Because of bad bugs in IRI2016 itself, present even in plain Fortran usage, we can't safely use F2py, bad data can result.

if os.name == 'nt':
    sfn = Path(__file__).parent / 'setup.cfg'
    stxt = sfn.read_text()
    if '[build_ext]' not in stxt:
        with sfn.open('a') as f:
            f.write("[build_ext]\ncompiler = mingw32")


src = [  # 'iriwebg.for',
    'irisub.for', 'irifun.for',
    'iritec.for', 'iridreg.for', 'igrf.for', 'cira.for', 'iriflip.for']

F = Path('src')
src = [str(F/s) for s in src]

ext = Extension(name='iri16', sources=src,
                f2py_options=['only:', 'iri_sub', ':'],  # ['skip:', 'dfridr', ':'],
                extra_f77_compile_args=['-w'])
"""

R = Path('iri2016') / 'data'
iridata = list(map(str,
                   (list((R/'ccir').glob('*.asc')) +
                    list((R/'igrf').glob('*.dat')) +
                       list((R/'index').glob('*.dat')) +
                       list((R/'mcsat').glob('*.dat')) +
                       list((R/'ursi').glob('*.asc')))
                   ))

iridata = list(map(str, iridata))  # even for Numpy 1.14 due to numpy.distutils

setuptools.setup(
    # ext_modules=[ext],
    data_files=iridata,
)

R = Path(__file__).parent
BINDIR = R / 'build'
SRCDIR = R / 'src'

cmakever = subprocess.check_output(['cmake', '--version'], universal_newlines=True)
cmakever = parse_version(cmakever.split()[2])
if cmakever < parse_version('3.13'):
    raise RuntimeError(
        'CMake >= 3.13 needed to build IRI2016.'
        'Please see https://cmake.org/download or https://github.com/scivision/cmake-utils/blob/master/cmake_setup.sh')

# %% workaround, CMake >= 3.13
if os.name == 'nt':
    subprocess.check_call(['cmake', '-G', 'MinGW Makefiles',
                           '-DCMAKE_SH="CMAKE_SH-NOTFOUND', '-S', str(SRCDIR), '-B', str(BINDIR)])
else:
    subprocess.check_call(['cmake', '-S', str(SRCDIR), '-B', str(BINDIR)])


subprocess.check_call(['cmake', '--build', str(BINDIR), '-j'])



import subprocess
from dateutil.parser import parse
from datetime import datetime, timedelta
from pathlib import Path
import os
import xarray
import io
import numpy as np
from typing import List, Sequence

BINDIR = Path(__file__).resolve().parents[1] / 'build'
EXE = './iri2016_driver'
SHELL = False
if os.name == 'nt':
    EXE = EXE[2:] + '.exe'
    SHELL = True

SIMOUT = ['ne', 'Tn', 'Ti', 'Te', 'nO+', 'nH+', 'nHe+', 'nO2+', 'nNO+',
          'nCI', 'nN+']


def datetimerange(start: datetime, end: datetime, step: timedelta) -> List[datetime]:
    """like range() for datetime"""
    if isinstance(start, str):
        start = parse(start)

    if isinstance(end, str):
        end = parse(end)

    assert isinstance(start, datetime)
    assert isinstance(end, datetime)
    assert isinstance(step, timedelta)

    return [start + i*step for i in range((end-start) // step)]


def IRI(time: datetime, altkmrange: Sequence[float],
        glat: float, glon: float) -> xarray.Dataset:

    if isinstance(time, str):
        time = parse(time)

    assert len(altkmrange) == 3, 'altitude (km) min, max, step'
    assert isinstance(glat, float) and isinstance(glon, float), 'glat, glon is scalar'

    # NOTE: Windows needs shell=True and str(pathlib.Path)
    cmd = [str(EXE),
           str(time.year), str(time.month), str(time.day),
           str(time.hour), str(time.minute), str(time.second),
           str(glat), str(glon),
           str(altkmrange[0]), str(altkmrange[1]), str(altkmrange[2])]

    ret = subprocess.check_output(cmd,
                                  universal_newlines=True, cwd=str(BINDIR),  # str for Windows
                                  shell=SHELL)
# %% get altitude profile data
    Nalt = int((altkmrange[1]-altkmrange[0]) // altkmrange[2]) + 1

    arr = np.genfromtxt(io.StringIO(ret), max_rows=Nalt)
    arr = np.atleast_2d(arr)
    assert arr.ndim == 2 and arr.shape[1] == 12, 'bad text data output format'

    dsf = {k: (('alt_km'), v) for (k, v) in zip(SIMOUT, arr[:, 1:].T)}
    altkm = arr[:, 0]
# %% get parameter data
    arr = np.genfromtxt(io.StringIO(ret), skip_header=Nalt)
    assert arr.ndim == 1 and arr.size == 100, 'bad text data output format'
# %% assemble output
    iono = xarray.Dataset(dsf,
                          coords={'time': [time], 'alt_km': altkm,
                                  'glat': glat, 'glon': glon},
                          attrs={'f107': arr[40], 'ap': arr[51]})

    for i, p in enumerate(['NmF2', 'hmF2', 'NmF1', 'hmF1', 'NmE', 'hmE']):
        iono[p] = (('time'), [arr[i]])

    iono['TEC'] = (('time'), [arr[36]])
    iono['EqVertIonDrift'] = (('time'), [arr[43]])

    return iono


def timeprofile(tlim: tuple, dt: timedelta,
                altkmrange: list, glat: float, glon: float) -> xarray.Dataset:
    """compute IRI altitude profile over time range for fixed lat/lon
    """

    T = datetimerange(tlim[0], tlim[1], dt)

    iono: xarray.Dataset = None

    f107 = []
    ap = []
    for t in T:
        iri = IRI(t, altkmrange, glat, glon)
        if iono is None:
            iono = iri
        else:
            iono = xarray.concat((iono, iri), dim='time')

        f107.append(iri.f107)
        ap.append(iri.ap)

    iono.attrs = iri.attrs
    iono.attrs['f107'] = f107
    iono.attrs['ap'] = ap

    return iono


def geoprofile(latrange: Sequence[float], glon: float,
               altkm: float,
               time: datetime) -> xarray.Dataset:
    """compute IRI altitude profiles at time, over lat or lon range
    """

    glat = np.arange(*latrange)

    iono: xarray.Dataset = None

    f107 = []
    ap = []
    for l in glat:
        iri = IRI(time, altkmrange=[altkm]*3, glat=l, glon=glon)
        if iono is None:
            iono = iri
        else:
            iono = xarray.concat((iono, iri), dim='glat')

        f107.append(iri.f107)
        ap.append(iri.ap)

    iono.attrs = iri.attrs
    iono.attrs['f107'] = f107
    iono.attrs['ap'] = ap

    return iono



#!/usr/bin/env python
from pathlib import Path
import subprocess
import pytest
import shutil

R = Path(__file__).parent


@pytest.mark.skipif(not shutil.which('matlab'), reason='matlab not found')
def test_matlab_api():
    subprocess.check_call(['matlab', '-nojvm', '-r',
                           'r=runtests(); exit(any([r.Failed]))'],
                          cwd=R, timeout=60)


@pytest.mark.skipif(not shutil.which('octave'), reason='octave not found')
def test_octave_api():
    subprocess.check_call(['octave', '-q', '--eval="exit(test_iri2016)"'],
                          cwd=R, timeout=60)


if __name__ == '__main__':
    pytest.main(['-xrsv', __file__])
